import { type RouteRecordRaw } from 'vue-router'
import { ViteSSG } from 'vite-ssg'
import { createPinia } from 'pinia'
import hljs from 'highlight.js'
import { marked } from 'marked'
import { markedHighlight } from 'marked-highlight'

import main from './main.vue'
import './main.sass'

import { createRoutes, useRouteStore } from './routes'
import { routes as appRoutes, header, type RouteDefinition } from 'content/routes.js'
import { headingSectionsExtension } from './utilities/marked'

marked
.use(headingSectionsExtension() as any)
.use(markedHighlight({
  langPrefix: 'hljs language-',
  highlight(code, lang) {
    const language = hljs.getLanguage(lang) ? lang : 'plaintext'
    return hljs.highlight(code, { language }).value
  },
}))

export const createApp = ViteSSG(
  // the root component
  main,
  // vue-router options
  { routes: createRoutes() },
  // function to have custom setups
  ({ app, router, routes, isClient, initialState }) => {
    app.use(createPinia())
    const routeStore = useRouteStore()
    Object.keys(appRoutes).forEach(route => 
      routeStore._routes[route] = {
        ...routes.find(other => other.path === route) as RouteRecordRaw,
        ...appRoutes[route] as RouteDefinition,
      }
    )
    routeStore._header = header
  },
)
